==> webkit2gtk_2.35.1.orig.tar.xz: webkitgtk-2.35.1/Source/WebCore/PAL/pal/graphics/WebGPU/Impl/WebGPUComputePassEncoderImpl.cpp <==
/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "WebGPUComputePassEncoderImpl.h"

#if HAVE(WEBGPU_IMPLEMENTATION)

#include "WebGPUBindGroupImpl.h"
#include "WebGPUBufferImpl.h"
#include "WebGPUComputePipelineImpl.h"
#include "WebGPUConvertToBackingContext.h"
#include "WebGPUQuerySetImpl.h"
#include <WebGPU/WebGPUExt.h>

namespace PAL::WebGPU {

ComputePassEncoderImpl::ComputePassEncoderImpl(WGPUComputePassEncoder computePassEncoder, ConvertToBackingContext& convertToBackingContext)
    : m_backing(computePassEncoder)
    , m_convertToBackingContext(convertToBackingContext)
{
}

ComputePassEncoderImpl::~ComputePassEncoderImpl()
{
    wgpuComputePassEncoderRelease(m_backing);
}

void ComputePassEncoderImpl::setPipeline(const ComputePipeline& computePipeline)
{
    wgpuComputePassEncoderSetPipeline(m_backing, m_convertToBackingContext->convertToBacking(computePipeline));
}

void ComputePassEncoderImpl::dispatch(Size32 x, std::optional<Size32> y, std::optional<Size32> z)
{
    wgpuComputePassEncoderDispatch(m_backing, x, y.value_or(1), z.value_or(1));
}

void ComputePassEncoderImpl::dispatchIndirect(const Buffer& indirectBuffer, Size64 indirectOffset)
{
    wgpuComputePassEncoderDispatchIndirect(m_backing, m_convertToBackingContext->convertToBacking(indirectBuffer), indirectOffset);
}

void ComputePassEncoderImpl::beginPipelineStatisticsQuery(const QuerySet& querySet, Size32 queryIndex)
{
    wgpuComputePassEncoderBeginPipelineStatisticsQuery(m_backing, m_convertToBackingContext->convertToBacking(querySet), queryIndex);
}

void ComputePassEncoderImpl::endPipelineStatisticsQuery()
{
    wgpuComputePassEncoderEndPipelineStatisticsQuery(m_backing);
}

void ComputePassEncoderImpl::endPass()
{
    wgpuComputePassEncoderEndPass(m_backing);
}

void ComputePassEncoderImpl::setBindGroup(Index32 index, const BindGroup& bindGroup,
    std::optional<Vector<BufferDynamicOffset>>&& offsets)
{
    auto backingOffsets = offsets.value_or(Vector<BufferDynamicOffset> { });
    wgpuComputePassEncoderSetBindGroup(m_backing, index, m_convertToBackingContext->convertToBacking(bindGroup), static_cast<uint32_t>(backingOffsets.size()), backingOffsets.data());
}

void ComputePassEncoderImpl::setBindGroup(Index32 index, const BindGroup& bindGroup,
    const uint32_t* dynamicOffsetsArrayBuffer,
    size_t dynamicOffsetsArrayBufferLength,
    Size64 dynamicOffsetsDataStart,
    Size32 dynamicOffsetsDataLength)
{
    UNUSED_PARAM(dynamicOffsetsArrayBufferLength);
    // FIXME: Use checked algebra.
    wgpuComputePassEncoderSetBindGroup(m_backing, index, m_convertToBackingContext->convertToBacking(bindGroup), dynamicOffsetsDataLength, dynamicOffsetsArrayBuffer + dynamicOffsetsDataStart);
}

void ComputePassEncoderImpl::pushDebugGroup(String&& groupLabel)
{
    wgpuComputePassEncoderPushDebugGroup(m_backing, groupLabel.utf8().data());
}

void ComputePassEncoderImpl::popDebugGroup()
{
    wgpuComputePassEncoderPopDebugGroup(m_backing);
}

void ComputePassEncoderImpl::insertDebugMarker(String&& markerLabel)
{
    wgpuComputePassEncoderInsertDebugMarker(m_backing, markerLabel.utf8().data());
}

void ComputePassEncoderImpl::setLabelInternal(const String& label)
{
    wgpuComputePassEncoderSetLabel(m_backing, label.utf8().data());
}

} // namespace PAL::WebGPU

#endif // HAVE(WEBGPU_IMPLEMENTATION)

==> webkit2gtk_2.35.2.orig.tar.xz: webkitgtk-2.35.2/Source/WebCore/PAL/pal/graphics/WebGPU/Impl/WebGPUComputePassEncoderImpl.cpp <==
/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "WebGPUComputePassEncoderImpl.h"

#if HAVE(WEBGPU_IMPLEMENTATION)

#include "WebGPUBindGroupImpl.h"
#include "WebGPUBufferImpl.h"
#include "WebGPUComputePipelineImpl.h"
#include "WebGPUConvertToBackingContext.h"
#include "WebGPUQuerySetImpl.h"
#include <WebGPU/WebGPUExt.h>

namespace PAL::WebGPU {

ComputePassEncoderImpl::ComputePassEncoderImpl(WGPUComputePassEncoder computePassEncoder, ConvertToBackingContext& convertToBackingContext)
    : m_backing(computePassEncoder)
    , m_convertToBackingContext(convertToBackingContext)
{
}

ComputePassEncoderImpl::~ComputePassEncoderImpl()
{
    wgpuComputePassEncoderRelease(m_backing);
}

void ComputePassEncoderImpl::setPipeline(const ComputePipeline& computePipeline)
{
    wgpuComputePassEncoderSetPipeline(m_backing, m_convertToBackingContext->convertToBacking(computePipeline));
}

void ComputePassEncoderImpl::dispatch(Size32 x, std::optional<Size32> y, std::optional<Size32> z)
{
    wgpuComputePassEncoderDispatch(m_backing, x, y.value_or(1), z.value_or(1));
}

void ComputePassEncoderImpl::dispatchIndirect(const Buffer& indirectBuffer, Size64 indirectOffset)
{
    wgpuComputePassEncoderDispatchIndirect(m_backing, m_convertToBackingContext->convertToBacking(indirectBuffer), indirectOffset);
}

void ComputePassEncoderImpl::beginPipelineStatisticsQuery(const QuerySet& querySet, Size32 queryIndex)
{
    wgpuComputePassEncoderBeginPipelineStatisticsQuery(m_backing, m_convertToBackingContext->convertToBacking(querySet), queryIndex);
}

void ComputePassEncoderImpl::endPipelineStatisticsQuery()
{
    wgpuComputePassEncoderEndPipelineStatisticsQuery(m_backing);
}

void ComputePassEncoderImpl::endPass()
{
    wgpuComputePassEncoderEndPass(m_backing);
}

void ComputePassEncoderImpl::setBindGroup(Index32 index, const BindGroup& bindGroup,
    std::optional<Vector<BufferDynamicOffset>>&& offsets)
{
    auto backingOffsets = valueOrDefault(offsets);
    wgpuComputePassEncoderSetBindGroup(m_backing, index, m_convertToBackingContext->convertToBacking(bindGroup), static_cast<uint32_t>(backingOffsets.size()), backingOffsets.data());
}

void ComputePassEncoderImpl::setBindGroup(Index32 index, const BindGroup& bindGroup,
    const uint32_t* dynamicOffsetsArrayBuffer,
    size_t dynamicOffsetsArrayBufferLength,
    Size64 dynamicOffsetsDataStart,
    Size32 dynamicOffsetsDataLength)
{
    UNUSED_PARAM(dynamicOffsetsArrayBufferLength);
    // FIXME: Use checked algebra.
    wgpuComputePassEncoderSetBindGroup(m_backing, index, m_convertToBackingContext->convertToBacking(bindGroup), dynamicOffsetsDataLength, dynamicOffsetsArrayBuffer + dynamicOffsetsDataStart);
}

void ComputePassEncoderImpl::pushDebugGroup(String&& groupLabel)
{
    wgpuComputePassEncoderPushDebugGroup(m_backing, groupLabel.utf8().data());
}

void ComputePassEncoderImpl::popDebugGroup()
{
    wgpuComputePassEncoderPopDebugGroup(m_backing);
}

void ComputePassEncoderImpl::insertDebugMarker(String&& markerLabel)
{
    wgpuComputePassEncoderInsertDebugMarker(m_backing, markerLabel.utf8().data());
}

void ComputePassEncoderImpl::setLabelInternal(const String& label)
{
    wgpuComputePassEncoderSetLabel(m_backing, label.utf8().data());
}

} // namespace PAL::WebGPU

#endif // HAVE(WEBGPU_IMPLEMENTATION)
